import java.util.Collection;
import java.util.ArrayList;

/**
 * Classe Banca - La classe simula il comportamento di una banca
 * I conti sono numerati automanticamente ad ogni apertura con 
 * un numero diverso.
 * La classe  utilizza un ArrayList come collezione
 * 
 * @author Mattia Demontis 
 * @version 1.0
 */
abstract public class Banca{
    protected Collection<ContoCorrente> conti;
    private String nomeBanca;
    private SequenzaDiNumeri sequenza;
    
    /**
     * Il costruttore crea un oggetto di classe Banca
     * Astratto perch non definisce la collezione
     * dove memorizzare i ContiCorrenti
     * @param nome nome della banca
     */
    public Banca(String nome){
        this.nomeBanca = nome;
        sequenza = new SequenzaDiNumeri(1);
        //conti = new ArrayList<ContoCorrente>();
    }
    
    /**
     * Cerca il conto con il numero indicato e lo restituisce.
     * Il numero del conto deve essere corretto
     * @param numero del conto
     * @return Conto con quel numero, null se non esiste
     * @todo generare IllegalArgumentException
     */
    private ContoCorrente getConto(int num){
       for(ContoCorrente c: conti){
          if (c.getNumeroCC()==num) return c; 
       }
       return null;
    }
      
    /**
     * Il metodo apriConto apre un nuovo conto
     * 
     * @param saldo numero reale che contiene il saldo da inserire
     * @return il numero del conto appena aperto.
     */
    public int apriConto(double saldo){
       int numeroCC = sequenza.next();
       conti.add(new ContoCorrente(numeroCC,saldo));
       return numeroCC;
    }
    
    /** 
     * Apre un conto con saldo 0.
     * @return il numero del conto appena aperto.
     */
    public int apriConto(){
        return apriConto(0.0);
    }
    
    /**
     * Cerca tra i conti quello con il numero corrispondente 
     * a quello preso in ingresso, se lo trova deposita sul 
     * conto l'importo fornito
     * @param numeroCC numero del conto
     * @param importo somma da depositare (>= 0)
     * @return true se il conto esiste; false se non esiste
     */
    public boolean deposita(int numeroCC, double importo){
        assert importo >=0 : "importo negativo non ammesso";
        ContoCorrente c = getConto(numeroCC);
        if( c == null) return false;
        c.deposita(importo);
        return true;
    }
    
    /**
     * Il metodo cerca tra i conti quello con il numero 
     * corrispondente a quello preso in ingresso,
     * se lo trova preleva dal conto l'importo preso in ingresso
     * 
     * @param numeroCC numero del conto
     * @param importo somma da prelevare (>= 0)
     * @return true se il conto viene trovato e l'operazione
     *  stata eseguita; false se non viene trovato
     */
    public boolean preleva(int numeroCC, double importo){
        assert importo >=0 : "importo negativo non ammesso";
        ContoCorrente c = getConto(numeroCC);
        if( c == null) return false;
        c.preleva(importo);
        return true;
    }
    
    /**
     * Il metodo cerca  i conti, se li trova, preleva dal 
     * primo l'importo e lo versa nel secondo conto
     * 
     * @param numeroCCda - numero conto da cui prelevare
     * @param numeroCCverso - numero conto su cui versare
     * @param importo la somma da spostare (>= 0)
     * @return true se l'operazione ha successo, 
     * false se almeno uno dei due conti non esiste 
     */
    public boolean giroconto(int numeroCcDa, int numeroCcVerso,
           double importo){
        assert importo >=0 : "importo negativo non ammesso";
        ContoCorrente cDa = getConto(numeroCcDa);
        if(cDa == null) return false;
        ContoCorrente cVerso = getConto(numeroCcVerso);
        if( cVerso == null) return false;
        cDa.preleva(importo);
        cVerso.deposita(importo);
        return true;
    }    
    
    /**
     * Somma il totale di tutti i saldi dei conti della banca
     * @return totale dei soldi depositati nella banca
     */
    public double getTotaleSoldiInBanca(){
        double tot=0.0;
        for(ContoCorrente c : conti){
            tot += c.getSaldo();
        }
        return tot;
    }
    
    /**
     * Fornisce il saldo del conto prescelto.
     * @param numeroCC numero del conto di cui sapere il saldo
     * @return saldo del conto; se non viene trovato 
     * il conto viene restituito 0.00
     * @todo IllegalArgumentException se numero cc non valido
     */
    public double getSaldo(int numeroCC){
        ContoCorrente c = getConto(numeroCC);
        if(c != null)
          return c.getSaldo();
        return 0.0;
    }
    
    /**
     * Preleva un importo preso in ingresso da tutti i conti.
     * @param importo somma da prelevare (>=0)
     * @return numero di conti su cui  stata fatta l'operazione
     * (indipendentemente da quanto prelevato, che potrebbe essere zero)
     */
    public int addSpesa(double importo){
        assert importo >= 0 :"Importo non negativo non ammesso";
        int i = 0;
        for(ContoCorrente c : conti){
            c.preleva(importo);
            i++;
        }
        return i;
    }
    
    /**
     * Fornisce il numero di conti presenti nella banca
     * @return numero di conti
     */
    public int nConti(){
        return conti.size();
    }
    
    /**
     * Chiude un conto corrente.
     * @param  numero del conto da chiudere
     * @return saldo finale del conto chiuso
     */
    public double chiudiConto(int numeroCC){
       double saldo = 0;
       ContoCorrente c = getConto(numeroCC);
       if (c!= null){
          saldo = c.getSaldo(); 
          conti.remove(c);
       }
       return saldo;
    }
    /**
     * Il metodo restituisce le informazioni relative alla banca 
     * sotto forma di stringa nel formato: 
     * "Sono presenti nConti conti;
     * il saldo totale  di totaleSoldiInBanca"
     *  
     * @return informazioni relative la banca
     */
    public String toString(){
        return "Banca "+ nomeBanca+ "\nSono presenti " + 
              nConti() + " conti; il saldo totale  di " + 
              getTotaleSoldiInBanca() + ".";
    }
}